//Contains hooks that control how rally points are used.

#include "rally_point.h"
#include "../SCBW/api.h"

//Helper functions declaration

namespace {

u32 function_004557C0(CUnit* unit, CUnit* target);														//557C0
Bool32 OrderAllowed(CUnit* unit, u16 order, u32 nationID);												//6DC20
bool isOrderValid(CUnit* unit, CUnit* target, u8 orderId, u32 unk);										//74D90
u32 getSpellStatString(CUnit* target, u32 techId, u8 playerId);											//91E80
bool unitIsActiveTransport(CUnit* unit);																//E6BA0

} //unnamed namespace

//Based on parts of right click logic
void SmartOrderAfterRally(CUnit* unit, CUnit* target, Point16 &pos) {

	u8 orderId;

	if (target != NULL) {

		if (units_dat::BaseProperty[unit->id] & UnitProperty::Worker) 
		{

			orderId = function_004557C0(unit, target);

			if (orderId == OrderId::None) {

				if (unit->id == UnitId::TerranSCV) {

					bool targetCannotNeedToBeBuilt = false;
					bool skippingAllLocalChecks = false;

					targetCannotNeedToBeBuilt =
						!(target->status & UnitStatus::GroundedBuilding) ||
						target->status & UnitStatus::Completed;

					if(!targetCannotNeedToBeBuilt)
						skippingAllLocalChecks =
							target->playerId != unit->playerId ||
							target->getRace() != RaceId::Terran ||
							units_dat::BaseProperty[target->id] & UnitProperty::Addon ||
							(target->connectedUnit != NULL && target->connectedUnit->orderTarget.unit == target);

					if (!targetCannotNeedToBeBuilt && !skippingAllLocalChecks)
						orderId = OrderId::ConstructingBuilding;
					else
					if (targetCannotNeedToBeBuilt) {

						if (!(playerAlliance[unit->playerId].flags[target->playerId] < 1)) {
							if(
								!(units_dat::BaseProperty[target->id] & UnitProperty::Building) &&
								unitIsActiveTransport(target) &&
								scbw::canBeEnteredBy(target,unit)
							)
								orderId = OrderId::EnterTransport;
							else
							if(isOrderValid(unit,target,OrderId::Repair1,0))
								orderId = OrderId::Repair1;
							else
							if(unitIsActiveTransport(target) && scbw::canBeEnteredBy(target,unit))
								orderId = OrderId::EnterTransport;
						}


					}

				}

				if (orderId == OrderId::None) {

					if(target->id >= UnitId::Special_ZergBeacon && target->id <= UnitId::Special_ProtossFlagBeacon)
						orderId = OrderId::Move;
					else
					if(units_dat::BaseProperty[target->id] & UnitProperty::NeutralAccessories)
						orderId = OrderId::Move;
					else
					if(unit->isTargetEnemy(target))
						orderId = OrderId::Attack1;
					else
					if(unitIsActiveTransport(target) && scbw::canBeEnteredBy(target,unit))
						orderId = OrderId::EnterTransport;
					else
					if(target->status & UnitStatus::Burrowed)
						orderId = OrderId::Move;
					else
						orderId = OrderId::Follow;

				}

			}

		}
		else
		if(target->id >= UnitId::Special_ZergBeacon && target->id <= UnitId::Special_ProtossFlagBeacon)
			orderId = OrderId::Move;
		else
		if(units_dat::BaseProperty[target->id] & UnitProperty::NeutralAccessories)
			orderId = OrderId::Move;
		else
		if (
			(unit->id == UnitId::ZergQueen || unit->id == UnitId::Hero_Matriarch) &&
			!(target->status & UnitStatus::Invincible) &&
			getSpellStatString(target,TechId::Infestation,unit->playerId) == 0
		)
			orderId = OrderId::InfestMine2;
		else
		if(unit->isTargetEnemy(target))
			orderId = OrderId::Attack1;
		else
		if(
			unit->id == UnitId::TerranMedic && 
			unit->playerId == target->playerId &&
			target->hitPoints < units_dat::MaxHitPoints[target->id]
		)
			orderId = OrderId::HealMove;
		else
		if(unitIsActiveTransport(target) && scbw::canBeEnteredBy(target,unit))
			orderId = OrderId::EnterTransport;
		else
		if(unitIsActiveTransport(unit) && isOrderValid(unit,target,OrderId::Pickup2,0))
			orderId = OrderId::Pickup2;
		else
		if(target->status & UnitStatus::Burrowed)
			orderId = OrderId::Move;
		else
			orderId = OrderId::Follow;

		if(orderId == OrderId::Attack1) {

			if (unit->subunit != NULL)
				unit->subunit->orderTo(units_dat::AttackUnitOrder[unit->subunit->id], target);

			orderId = units_dat::AttackUnitOrder[unit->id];

		}

		if (OrderAllowed(unit, orderId, unit->playerId))
			unit->orderTo(orderId, target);
		else
			unit->orderTo(OrderId::Move, target->getX(), target->getY());

	}
	else {
		orderId = OrderId::Move;
		unit->orderTo(orderId, pos.x, pos.y);
	}

}

namespace hooks {

/// Orders newly-produced units to rally, based upon the properties of the
/// building that produced it.
///
/// @param  unit      The unit that needs to receive rally orders.
/// @param  factory   The unit (building) that created the given unit.
/// 
/// For lurker egg and cocoon, factory == unit

void orderNewUnitToRally(CUnit* unit, CUnit* factory) {


	if(unit->rally.unit != NULL) {
		SmartOrderAfterRally(unit, unit->rally.unit, unit->rally.pt);
	}
	else
	if(unit->rally.pt.x != 0 || unit->rally.pt.y != 0)
		SmartOrderAfterRally(unit, NULL, unit->rally.pt);
	else
	//Factory do nothing if the rally target is the factory itself or the rally target position is 0
	if (
		factory != NULL && 
		factory->isFactory() && 
		factory->rally.unit != factory && 
		factory->rally.pt.x != 0
	)
		SmartOrderAfterRally(unit, factory->rally.unit, factory->rally.pt);

}

;

/// Called when the player sets the rally point on the ground.
void setRallyPosition(CUnit* unit, u16 x, u16 y) {
	unit->rally.unit = NULL;
	unit->rally.pt.x = x;
	unit->rally.pt.y = y;
}

;

/// Called when the player sets the rally point on a unit.
void setRallyUnit(CUnit* unit, CUnit* target) {

	if (target == NULL) 
		target = unit;

	unit->rally.unit = target;
	unit->rally.pt.x = target->getX();
	unit->rally.pt.y = target->getY();

}

;

} //hooks

//-------- Helper function definitions. Do NOT modify! --------//

namespace {

const u32 Func_Sub557C0 = 0x004557C0;
u32 function_004557C0(CUnit* unit, CUnit* target) {

	static u32 return_value;

	__asm {
		PUSHAD
		MOV EDI, target
		MOV ESI, unit
		CALL Func_Sub557C0
		MOV return_value, EAX
		POPAD
	}

	return return_value;

}

;

const u32 Func_OrderAllowed = 0x0046DC20;
Bool32 OrderAllowed(CUnit* unit, u16 order, u32 nationID) {

	static Bool32 bResult;

	__asm {
		PUSHAD
		MOV BX, order
		MOV EAX, unit
		PUSH nationID
		CALL Func_OrderAllowed
		MOV bResult, EAX
		POPAD
	}

	return bResult;

}

;

const u32 Func_isOrderValid = 0x00474D90;
bool isOrderValid(CUnit* unit, CUnit* target, u8 orderId, u32 unk) {

	static Bool32 bPreResult;

	__asm {
		PUSHAD
		PUSH unk
		MOV EBX, unit
		MOV ESI, target
		MOV AL, orderId
		CALL Func_isOrderValid
		MOV bPreResult, EAX
		POPAD
	}

	return (bPreResult != 0);

}

;

const u32 Func_getSpellStatString = 0x00491E80;
u32 getSpellStatString(CUnit* target, u32 techId, u8 playerId) {

	static u32 return_value;

	__asm {
		PUSHAD
		MOV BL, playerId
		PUSH techId
		MOV EAX, target
		CALL Func_getSpellStatString
		MOV return_value, EAX
		POPAD
	}

	return return_value;
}

;

//Equivalent to code unitIsActiveTransport @ 0x004E6BA0
bool unitIsActiveTransport(CUnit* unit) {

	bool result = false;

	if(!(unit->status & UnitStatus::IsHallucination)) {

		if(
			unit->id != UnitId::ZergOverlord ||
			UpgradesSc->currentLevel[unit->playerId][UpgradeId::VentralSacs] != 0
		)
		{
			if(units_dat::SpaceProvided[unit->id] != 0)
				result = true;
		}


	}

	return result;

}

;

} //unnamed namespace

//End of helper functions