#include "CMDRECV_LiftOff.h"
#include <SCBW/api.h>

//Helper functions declaration

namespace {

bool unitIsTrainingOrMorphing(CUnit* unit);															//01500
bool unitIsGhost(CUnit* unit);																		//01570
void parseOrdersDatReqs();																			//6D450
int parseRequirementOpcodes(CUnit* unit, u32 orderId, u32 playerId, u32 address, u32 req_offset);	//6D610
void _doOrder(CUnit* unit, u32 orderId, int x, int y, bool stopPreviousOrders);						//74C70

} //unnamed namespace

namespace hooks {

void CMDRECV_Liftoff(s16 x, s16 y) {

	u16* const maxBoxRightValue =	(u16*)0x00628450;	//should usually be mapTileSize->width * 32
	u16* const maxBoxBottomValue =	(u16*)0x006284B4;	//should usually be mapTileSize->height * 32
	u32* const u32_0066FF60 =		(u32*)0x0066FF60;

	CUnit* currentUnit;
	CUnit* mainSelectedUnit;
	
	*selectionIndexStart = 0;

	currentUnit = getActivePlayerNextSelection();
	mainSelectedUnit = currentUnit;

	while(
		currentUnit != NULL &&
		!unitIsTrainingOrMorphing(currentUnit) &&
		currentUnit->building.techType == TechId::None &&
		currentUnit->building.upgradeType == UpgradeId::None &&
		currentUnit->status & UnitStatus::GroundedBuilding &&
		units_dat::BaseProperty[currentUnit->id] & UnitProperty::FlyingBuilding
	)
	{

		s16 current_x, current_y;

		if(currentUnit != mainSelectedUnit){
			current_x = x + (mainSelectedUnit->getX() - currentUnit->getX());
			current_y = y + (mainSelectedUnit->getY() - currentUnit->getY());
		}
		else
		{
			current_x = x;
			current_y = y;
		}


		*u32_0066FF60 = 0;

		if (orders_dat::RequirementsOffset[OrderId::BuildingLiftoff] == 0xFFFF)
			parseOrdersDatReqs();

		if (currentUnit->playerId != *ACTIVE_NATION_ID)
			*u32_0066FF60 = 0x01;
		else
		if(!(currentUnit->status & UnitStatus::Completed))
			*u32_0066FF60 = 0x14;
		else
		if (
			currentUnit->status & UnitStatus::DoodadStatesThing ||
			currentUnit->lockdownTimer != 0 ||
			currentUnit->stasisTimer != 0 ||
			currentUnit->maelstromTimer != 0
		)
			*u32_0066FF60 = 0x0A;
		else
		if(currentUnit->status & UnitStatus::Burrowed && currentUnit->id != UnitId::ZergLurker)
			*u32_0066FF60 = 0x0B;
		else
		if(
			(currentUnit->id == UnitId::TerranSCV && currentUnit->mainOrderId == OrderId::ConstructingBuilding) ||
			(unitIsGhost(currentUnit) && currentUnit->mainOrderId == OrderId::NukeTrack) ||
			(currentUnit->id == UnitId::ProtossArchon && currentUnit->mainOrderId == OrderId::CompletingArchonSummon) ||
			(currentUnit->id == UnitId::ProtossDarkArchon && currentUnit->mainOrderId == OrderId::CompletingArchonSummon)
		)
			*u32_0066FF60 = 0x08;
		else
		if(orders_dat::RequirementsOffset[OrderId::BuildingLiftoff] == 0)
			*u32_0066FF60 = 0x17;
		else
		if(
			parseRequirementOpcodes(
				currentUnit,
				OrderId::BuildingLiftoff,
				*ACTIVE_NATION_ID,
				0x00514CF8,
				orders_dat::RequirementsOffset[OrderId::BuildingLiftoff]
			) &&
			current_x < *maxBoxRightValue && current_y < *maxBoxBottomValue
		)
		{
			currentUnit->orderTo(OrderId::BuildingLiftoff, current_x, current_y);
			_doOrder(currentUnit, units_dat::ReturnToIdleOrder[currentUnit->id], current_x, current_y - 42, false);
		}

		currentUnit = getActivePlayerNextSelection();

	}

}

;

} //namespace hooks

//-------- Helper function definitions. Do NOT modify! --------//

namespace {

//Identical to unitIsTrainingOrMorphing @ 0x00401500
bool unitIsTrainingOrMorphing(CUnit* unit) {

	bool bReturnValue;

	if(unit->buildQueue[unit->buildQueueSlot % 5] != UnitId::None)
		bReturnValue = true;
	else
	if(
		unit->secondaryOrderId == OrderId::BuildAddon &&
		unit->status & UnitStatus::GroundedBuilding &&
		unit->currentBuildUnit != NULL &&
		!(unit->currentBuildUnit->status & UnitStatus::Completed)
		)
		bReturnValue = true;
	else
	if(
		unit->mainOrderId != OrderId::BuildProtoss1 ||
		!(unit->status & UnitStatus::GroundedBuilding) ||
		unit->orderTarget.unit == NULL
		)
		bReturnValue = false;
	else
	if(!(unit->orderTarget.unit->status & UnitStatus::Completed))
		bReturnValue = true;
	else
		bReturnValue = false;

	return bReturnValue;

}

;

//Identical to unitIsGhost @ 0x00401570
bool unitIsGhost(CUnit* unit) {

	bool return_value;
	
	if (
		unit->id == UnitId::TerranGhost ||
		unit->id == UnitId::Hero_SarahKerrigan ||
		unit->id == UnitId::Hero_AlexeiStukov ||
		unit->id == UnitId::Hero_SamirDuran ||
		unit->id == UnitId::Hero_InfestedDuran
	)
		return_value = true;
	else
		return_value = false;

	return return_value;

}

;

const u32 Func_parseOrdersDatReqs = 0x0046D450;
void parseOrdersDatReqs() {

	__asm {
		PUSHAD
		CALL Func_parseOrdersDatReqs
		POPAD
	}

}

;

const u32 Func_parseRequirementOpcodes = 0x0046D610;
int parseRequirementOpcodes(CUnit* unit, u32 orderId, u32 playerId, u32 address, u32 req_offset) {

	static int return_value;

	__asm {
		PUSHAD
		MOV EAX, req_offset
		MOV ESI, unit
		PUSH address
		PUSH playerId
		PUSH orderId
		CALL Func_parseRequirementOpcodes
		MOV return_value, EAX
		POPAD
	}

	return return_value;

}

;

const u32 Func__doOrder = 0x00474C70;
void _doOrder(CUnit* unit, u32 orderId, int x, int y, bool stopPreviousOrders) {

	Bool32 bStopPreviousOrders = stopPreviousOrders ? 1 : 0;

	__asm {
		PUSHAD
		PUSH bStopPreviousOrders
		PUSH y
		PUSH x
		PUSH orderId
		PUSH unit
		CALL Func__doOrder
		POPAD
	}

}

;

} //unnamed namespace

//End of helper functions