#pragma once
#include <logger.h>

#define WRITE_TO_LOG(x) GPTP::logger<<x<<std::endl

namespace {

	const u32 Func_GetAllUnitsInBounds = 0x0042FF80;
	CUnit** getAllUnitsInBounds(Box16* coords) {

		static CUnit** units_in_bounds;

		__asm {
			PUSHAD
			MOV EAX, coords
			CALL Func_GetAllUnitsInBounds
			MOV units_in_bounds, EAX
			POPAD
		}

		return units_in_bounds;

	}

	;

}

const int MAX_SIGHT_RANGE = 11; //should be made equal to value of hooks\unit_stats\sight_range.cpp if it is modded

//Custom function to check if an unit can move, according to
//criterias relevant for this use case
bool canMove(CUnit* unit) {

	bool bResult;

	if(unit != NULL) {
		if(unit->id == UnitId::ZergLurker && unit->status & UnitStatus::Burrowed)
			bResult = false;
		else {
			if(units_dat::RightClickAction[unit->id] == RightClickActions::NoCommand_AutoAttack) {
				if(unit->status & UnitStatus::GroundedBuilding)
					bResult = false;
				else
					bResult = unit->isFactory();
			}
			else
				bResult = units_dat::RightClickAction[unit->id] != RightClickActions::NoMove_NormalAttack;
		}
	}
	else
		bResult = false;

	return bResult;

}

;

//Draw a line between the selected unit and
//the targeted rally location/unit.
//Draw a dot at destination.
//Color is the selected unit color.
inline void DrawRallyPoint(CUnit *selectedUnit) {

	u16 xdest, ydest;

	if (selectedUnit->id == UnitId::ZergNydusCanal && selectedUnit->building.nydusExit != NULL) {
		graphics::drawLine(
			selectedUnit->position.x,
			selectedUnit->position.y,
			selectedUnit->building.nydusExit->position.x,
			selectedUnit->building.nydusExit->position.y,
			selectedUnit->getColor(),
			graphics::CoordType::ON_MAP);

		graphics::drawFilledCircle(
			selectedUnit->building.nydusExit->position.x,
			selectedUnit->building.nydusExit->position.y,
			3,
			selectedUnit->getColor(),
			graphics::CoordType::ON_MAP);
	}
	else
	if(selectedUnit->id == UnitId::ZergEgg || selectedUnit->id == UnitId::ZergLurkerEgg || selectedUnit->id == UnitId::ZergCocoon) {

		if (selectedUnit->rally.unit != NULL) {

			xdest = selectedUnit->rally.unit->position.x;
			ydest = selectedUnit->rally.unit->position.y;

			graphics::drawLine(
				selectedUnit->position.x,
				selectedUnit->position.y,
				xdest,
				ydest,
				selectedUnit->getColor(),
				graphics::CoordType::ON_MAP);

			graphics::drawFilledCircle(
				xdest,
				ydest,
				3,
				selectedUnit->getColor(),
				graphics::CoordType::ON_MAP);

		}
		else
		if(selectedUnit->rally.pt.x != 0 || selectedUnit->rally.pt.y != 0) {

			xdest = selectedUnit->rally.pt.x;
			ydest = selectedUnit->rally.pt.y;

			if(xdest != selectedUnit->position.x && ydest != selectedUnit->position.y)
			{

				graphics::drawLine(
					selectedUnit->position.x,
					selectedUnit->position.y,
					xdest,
					ydest,
					selectedUnit->getColor(),
					graphics::CoordType::ON_MAP);

				graphics::drawFilledCircle(
					xdest,
					ydest,
					3,
					selectedUnit->getColor(),
					graphics::CoordType::ON_MAP);

			}

		}
		else
		if(selectedUnit->id == UnitId::ZergEgg && selectedUnit->connectedUnit != NULL) {

			if(selectedUnit->connectedUnit->rally.unit == NULL) {

				xdest = selectedUnit->connectedUnit->rally.pt.x;
				ydest = selectedUnit->connectedUnit->rally.pt.y;

				if(
					(xdest != 0 || ydest != 0) &&
					(xdest != selectedUnit->connectedUnit->position.x && ydest != selectedUnit->connectedUnit->position.y)
					)
				{

					graphics::drawDottedLine(
						selectedUnit->connectedUnit->position.x,
						selectedUnit->connectedUnit->position.y,
						selectedUnit->position.x,
						selectedUnit->position.y,
						selectedUnit->getColor(),
						graphics::CoordType::ON_MAP);

					graphics::drawLine(
						selectedUnit->position.x,
						selectedUnit->position.y,
						xdest,
						ydest,
						selectedUnit->getColor(),
						graphics::CoordType::ON_MAP);

					graphics::drawFilledCircle(
						xdest,
						ydest,
						3,
						selectedUnit->getColor(),
						graphics::CoordType::ON_MAP);

				}

			}
			else
			if(selectedUnit->connectedUnit->rally.unit != selectedUnit->connectedUnit) 
			{

				xdest = selectedUnit->connectedUnit->rally.unit->position.x;
				ydest = selectedUnit->connectedUnit->rally.unit->position.y;

				graphics::drawDottedLine(
					selectedUnit->connectedUnit->position.x,
					selectedUnit->connectedUnit->position.y,
					selectedUnit->position.x,
					selectedUnit->position.y,
					selectedUnit->getColor(),
					graphics::CoordType::ON_MAP);

				graphics::drawLine(
					selectedUnit->position.x,
					selectedUnit->position.y,
					xdest,
					ydest,
					selectedUnit->getColor(),
					graphics::CoordType::ON_MAP);

				graphics::drawFilledCircle(
					xdest,
					ydest,
					3,
					selectedUnit->getColor(),
					graphics::CoordType::ON_MAP);

			}

		}

	}
	else
	if(selectedUnit->rally.unit == NULL) {

		xdest = selectedUnit->rally.pt.x;
		ydest = selectedUnit->rally.pt.y;

		if(
			(xdest != 0 || ydest != 0) &&
			(xdest != selectedUnit->position.x && ydest != selectedUnit->position.y)
			)
		{

			graphics::drawLine(
				selectedUnit->position.x,
				selectedUnit->position.y,
				xdest,
				ydest,
				selectedUnit->getColor(),
				graphics::CoordType::ON_MAP);

			graphics::drawFilledCircle(
				xdest,
				ydest,
				3,
				selectedUnit->getColor(),
				graphics::CoordType::ON_MAP);

		}

	}
	else
	if(selectedUnit->rally.unit != selectedUnit) 
	{

		xdest = selectedUnit->rally.unit->position.x;
		ydest = selectedUnit->rally.unit->position.y;

		graphics::drawLine(
			selectedUnit->position.x,
			selectedUnit->position.y,
			xdest,
			ydest,
			selectedUnit->getColor(),
			graphics::CoordType::ON_MAP);

		graphics::drawFilledCircle(
			xdest,
			ydest,
			3,
			selectedUnit->getColor(),
			graphics::CoordType::ON_MAP);

	}

}

;

//Draw a line between the selected unit and
//the target of its order.
//Draw an ellipse around the destination unit/location.
//Color/size of ellipse is based on color/size of target unit,
//or color/size of selected unit if target is a location.
inline void DrawTargetedOrder(CUnit *selectedUnit) {

#define GET_TARGET_WIDTH(x) (u8)x->sprite->mainGraphic->grpOffset->frames[x->sprite->mainGraphic->frameIndex].width
#define GET_TARGET_HEIGHT(x) (u8)x->sprite->mainGraphic->grpOffset->frames[x->sprite->mainGraphic->frameIndex].height

	if(selectedUnit->orderTarget.unit != NULL) {

		if(selectedUnit->orderTarget.unit != selectedUnit) {

			graphics::drawLine(
				selectedUnit->position.x,
				selectedUnit->position.y,
				selectedUnit->orderTarget.unit->position.x, 
				selectedUnit->orderTarget.unit->position.y,
				selectedUnit->getColor(),
				graphics::CoordType::ON_MAP
			);

			graphics::drawEllipse(
				selectedUnit->orderTarget.unit->getLeft(),
				selectedUnit->orderTarget.unit->getTop(),
				selectedUnit->orderTarget.unit->getRight(),
				selectedUnit->orderTarget.unit->getBottom(),
				selectedUnit->orderTarget.unit->getColor(),
				graphics::CoordType::ON_MAP
			);

		}

	}else
	if(selectedUnit->orderTarget.pt.x != 0 || selectedUnit->orderTarget.pt.y != 0) {

		if(canMove(selectedUnit) &&
			(selectedUnit->orderTarget.pt.x != selectedUnit->position.x &&
				selectedUnit->orderTarget.pt.y != selectedUnit->position.y)
		) 
		{

			graphics::drawLine(
				selectedUnit->position.x,
				selectedUnit->position.y,
				selectedUnit->orderTarget.pt.x,
				selectedUnit->orderTarget.pt.y,
				selectedUnit->getColor(),
				graphics::CoordType::ON_MAP
			);

			graphics::drawEllipse(
				selectedUnit->orderTarget.pt.x - ( GET_TARGET_WIDTH(selectedUnit) / 2),
				selectedUnit->orderTarget.pt.y - ( GET_TARGET_HEIGHT(selectedUnit) / 2),
				selectedUnit->orderTarget.pt.x + ( GET_TARGET_WIDTH(selectedUnit) / 2),
				selectedUnit->orderTarget.pt.y + ( GET_TARGET_HEIGHT(selectedUnit) / 2),
				selectedUnit->getColor(),
				graphics::CoordType::ON_MAP
			);

		}

	}else
	if(canMove(selectedUnit)) {

		if(selectedUnit->moveTarget.unit != NULL) {

			if(selectedUnit->moveTarget.unit != selectedUnit) {

				graphics::drawLine(
					selectedUnit->position.x,
					selectedUnit->position.y,
					selectedUnit->moveTarget.unit->position.x,
					selectedUnit->moveTarget.unit->position.y,
					selectedUnit->getColor(),
					graphics::CoordType::ON_MAP
				);

				graphics::drawEllipse(
					selectedUnit->moveTarget.unit->getLeft(),
					selectedUnit->moveTarget.unit->getTop(),
					selectedUnit->moveTarget.unit->getRight(),
					selectedUnit->moveTarget.unit->getBottom(),
					selectedUnit->moveTarget.unit->getColor(),
					graphics::CoordType::ON_MAP
				);

			}

		}
		else
		if(selectedUnit->moveTarget.pt.x != 0 || selectedUnit->moveTarget.pt.y != 0) {

			if(	selectedUnit->moveTarget.pt.x != selectedUnit->position.x && 
				selectedUnit->moveTarget.pt.y != selectedUnit->position.y
				)
			{

				graphics::drawLine(
					selectedUnit->position.x,
					selectedUnit->position.y,
					selectedUnit->moveTarget.pt.x,
					selectedUnit->moveTarget.pt.y,
					selectedUnit->getColor(),
					graphics::CoordType::ON_MAP
				);

				graphics::drawEllipse(
					selectedUnit->moveTarget.pt.x - ( GET_TARGET_WIDTH(selectedUnit) / 2),
					selectedUnit->moveTarget.pt.y - ( GET_TARGET_HEIGHT(selectedUnit) / 2),
					selectedUnit->moveTarget.pt.x + ( GET_TARGET_WIDTH(selectedUnit) / 2),
					selectedUnit->moveTarget.pt.y + ( GET_TARGET_HEIGHT(selectedUnit) / 2),
					selectedUnit->getColor(),
					graphics::CoordType::ON_MAP
				);

			}

		}

	} //if(selectedUnit->canMove())

#undef GET_TARGET_WIDTH
#undef GET_TARGET_HEIGHT

}

;

//Draw dotted lines between queued orders,
//and from current move destination.
//For SCV constructing, the line tie the SCV
//to where it will go after constructing.
//Color is selected unit color.
inline void DrawQueuedOrders(CUnit *selectedUnit) {

	COrder* current_order = selectedUnit->orderQueueHead;

			//graphics::ColorId color;

			//if (
			//	(
			//		selectedUnit->orderTarget.unit->id >= UnitId::ResourceMineralField &&
			//		selectedUnit->orderTarget.unit->id <= UnitId::ResourceMineralFieldType3
			//	) ||
			//	selectedUnit->orderTarget.unit->id == UnitId::ResourceVespeneGeyser
			//)
			//	color = graphics::CYAN;
			//else
			//	color = ;

	if(current_order != NULL) {

		WRITE_TO_LOG("found a queued order");

		//SCV destination while constructing
		if(current_order->target.pt.x == 0 && current_order->target.pt.y == 0) {

			int x, y;

			do {
				current_order = current_order->next;
			} while (
				current_order != NULL && 
				current_order->target.pt.x == 0 && current_order->target.pt.y == 0 &&
				current_order->target.unit == NULL
			);

			if (current_order != NULL)
			{

				if (current_order->target.unit == NULL) {
					x = current_order->target.pt.x;
					y = current_order->target.pt.y;
				}
				else {
					x = current_order->target.unit->getX();
					y = current_order->target.unit->getY();
				}

				graphics::drawDottedLine(
					selectedUnit->position.x,
					selectedUnit->position.y,
					x,
					y,
					selectedUnit->getColor(),
					graphics::CoordType::ON_MAP);

				current_order = current_order->next;

				if (
					current_order != NULL &&
					current_order->target.pt.x == 0 && current_order->target.pt.y == 0 &&
					current_order->target.unit == NULL
				)
					current_order = NULL;

			}

			while (current_order != NULL)
			{

				if (current_order->target.unit == NULL) {

					graphics::drawDottedLine(
						x,
						y,
						current_order->target.pt.x,
						current_order->target.pt.y,
						selectedUnit->getColor(),
						graphics::CoordType::ON_MAP);

					x = current_order->target.pt.x;
					y = current_order->target.pt.y;

				}
				else
				{

					graphics::drawDottedLine(
						x,
						y,
						current_order->target.unit->sprite->position.x,
						current_order->target.unit->sprite->position.y,
						selectedUnit->getColor(),
						graphics::CoordType::ON_MAP);

					x = current_order->target.unit->sprite->position.x;
					y = current_order->target.unit->sprite->position.y;

				}

				current_order = current_order->next;

				if (
					current_order != NULL &&
					current_order->target.pt.x == 0 && current_order->target.pt.y == 0 &&
					current_order->target.unit == NULL
				)
					current_order = NULL;

			}

		}
		else {

			//line between current move destination and first queued order
			graphics::drawDottedLine(
				selectedUnit->moveTarget.pt.x,
				selectedUnit->moveTarget.pt.y,
				current_order->target.pt.x,
				current_order->target.pt.y,
				selectedUnit->getColor(),
				graphics::CoordType::ON_MAP);

			current_order = current_order->next;

			while(current_order != NULL && ( current_order->target.pt.x != 0 && current_order->target.pt.y != 0 ) ) {

				//line between previous order destination and current one
				graphics::drawDottedLine(
					current_order->prev->target.pt.x,
					current_order->prev->target.pt.y,
					current_order->target.pt.x,
					current_order->target.pt.y,
					selectedUnit->getColor(),
					graphics::CoordType::ON_MAP);

				current_order = current_order->next;

			}

		}

	}

}

;

//Try to display range of an unit as a dotted ellipse.
//Protoss carriers and reavers show range of
//their subunits.
//For medics (with energy) it's the healing range.
//For bunkers, display the range of the highest range
//unit in the bunker.
//For others, try to display their range, using the
//smaller one if they have both ground and air weapons.
//The color is the unit color.
inline void TryDrawMinWeaponRange(CUnit *selectedUnit) {

	if(	selectedUnit->id == UnitId::carrier || 
		selectedUnit->id == UnitId::gantrithor ||
		selectedUnit->id == UnitId::reaver ||
		selectedUnit->id == UnitId::warbringer
		) 
	{

		u16 range = selectedUnit->getSeekRange() * 32;

		if(	selectedUnit->carrier.inHangarCount > 0 ||
			selectedUnit->carrier.outHangarCount > 0)
			graphics::drawDottedEllipse(
				selectedUnit->position.x - range,
				selectedUnit->position.y - range,
				selectedUnit->position.x + range,
				selectedUnit->position.y + range,
				selectedUnit->getColor(),
				graphics::CoordType::ON_MAP
			);

	}
	else
	if(selectedUnit->id == UnitId::medic) {

		if(selectedUnit->energy > 0) 
#define HEAL_RANGE *(u8*)(0x00463cac) //should be 30
			graphics::drawDottedEllipse(
				selectedUnit->position.x - HEAL_RANGE,
				selectedUnit->position.y - HEAL_RANGE,
				selectedUnit->position.x + HEAL_RANGE,
				selectedUnit->position.y + HEAL_RANGE,
				selectedUnit->getColor(),
				graphics::CoordType::ON_MAP
			);
#undef HEAL_RANGE

	}
	else {

		if (selectedUnit->id != UnitId::TerranBunker) {

			u32 range_ground;
			u32 range_air;
			u32 range_drawn;

			range_ground = selectedUnit->getGroundWeapon();

			if (range_ground == WeaponId::None && selectedUnit->subunit != NULL)
				range_ground = selectedUnit->subunit->getGroundWeapon();

			if (range_ground != WeaponId::None)
				range_ground = selectedUnit->getMaxWeaponRange(range_ground);
			else
				range_ground = UINT_MAX;

			range_air = selectedUnit->getAirWeapon();

			if (range_air == WeaponId::None && selectedUnit->subunit != NULL)
				range_air = selectedUnit->subunit->getAirWeapon();

			if (range_air != WeaponId::None)
				range_air = selectedUnit->getMaxWeaponRange(range_air);
			else
				range_air = UINT_MAX;

			range_drawn = MIN(range_ground, range_air);

			if (range_drawn != UINT_MAX)
				graphics::drawDottedEllipse(
					selectedUnit->position.x - range_drawn,
					selectedUnit->position.y - range_drawn,
					selectedUnit->position.x + range_drawn,
					selectedUnit->position.y + range_drawn,
					selectedUnit->getColor(),
					graphics::CoordType::ON_MAP
				);

		}
		else
		{

			u16 bestRange = 0;

			for (int i = 0; i < 8; i++) {
				if (selectedUnit->loadedUnit[i].innerValues.index != 0) {
					u8 loaded_unit_weapon = (selectedUnit->getLoadedUnit(i))->getGroundWeapon();
					if(loaded_unit_weapon != WeaponId::None)
						bestRange = MAX(selectedUnit->getMaxWeaponRange(loaded_unit_weapon), bestRange);
				}
			}

			if(bestRange != 0)
				graphics::drawDottedEllipse(
					selectedUnit->position.x - bestRange,
					selectedUnit->position.y - bestRange,
					selectedUnit->position.x + bestRange,
					selectedUnit->position.y + bestRange,
					selectedUnit->getColor(),
					graphics::CoordType::ON_MAP
				);

		}

	}

}
;

//Draw a dotted white circle around a detector unit
//This is not perfect as cloaked units bounds are
//not obvious enough to be perfectly used along this
//circle
void DrawDetectorRange(CUnit* detector) {

	u32 sightRange = detector->getSightRange(false);

	graphics::drawDottedEllipse2(
		detector->getLeft() - sightRange * 32 - 1,
		detector->getTop() - sightRange * 32 - 1,
		detector->getRight() + sightRange * 32 + 1,
		detector->getBottom() + sightRange * 32 + 1,
		graphics::WHITE,
		detector->getColor(),
		graphics::CoordType::ON_MAP
	);

}

//Draw a green dotted line between the reference flying
//building and its currently being decided (with the
//"construction" interface) position to land (since
//other selected buildings would be positioned
//depending on that)
//Also for nydus canal making their exit (display the one
//that will build its exit, not grouped and relative)
inline void TryDrawFlyingBuildingPathToLandingPoint() {

#define UNIT (*activePortraitUnit)

	if (*IS_PLACING_BUILDING)
	{
		if (units_dat::BaseProperty[UNIT->id] & UnitProperty::FlyingBuilding) {
			graphics::drawDottedLine(
				UNIT->getX(),
				UNIT->getY(),
				*screenX + mouse->x,
				*screenY + mouse->y,
				graphics::GREEN,
				graphics::ON_MAP
			);
		}
		else
		if(UNIT->id == UnitId::ZergNydusCanal)
		{

			int i = 0;
			CUnit* builder = clientSelectionGroup->unit[i];

			while (
				i < SELECTION_ARRAY_LENGTH &&
				(
					builder == NULL ||
					builder->building.nydusExit != NULL
				)
			)
			{
				i++;
				builder = clientSelectionGroup->unit[i];
			}

			if(builder != NULL)
				graphics::drawDottedLine(
					builder->getX(),
					builder->getY(),
					*screenX + mouse->x,
					*screenY + mouse->y,
					graphics::GREEN,
					graphics::ON_MAP
				);
		}

	}

#undef UNIT 

}

namespace modding {

	//Draw rally points, movements toward orders,weapon minimum
	//range for some units, and visible enemy detectors range 
	//if cloaked units are within selection
	void drawRallyAndOthersForSelectedUnits() {

		CUnit* selectedUnit;
		Box16 coords;

		coords.left = mapTileSize->width * 32;
		coords.top = mapTileSize->height * 32;
		coords.right = 0;
		coords.bottom = 0;

		for (int i = 0; i < SELECTION_ARRAY_LENGTH; i++) {

			selectedUnit = clientSelectionGroup->unit[i];

			if(selectedUnit != NULL && (scbw::isInReplay() || scbw::isAlliedTo(selectedUnit->playerId,*LOCAL_HUMAN_ID) )) {

				if(selectedUnit->status & UnitStatus::InAir && selectedUnit->id != UnitId::ZergCocoon)											//Terran Flying Buildings especially
					DrawTargetedOrder(selectedUnit);
				else
				if(																						//Rally point case
					selectedUnit->isFactory() ||
					selectedUnit->id == UnitId::ZergNydusCanal ||
					selectedUnit->id == UnitId::ZergEgg || 
					selectedUnit->id == UnitId::ZergLurkerEgg || 
					selectedUnit->id == UnitId::ZergCocoon
				)
					DrawRallyPoint(selectedUnit);
				else
				if(selectedUnit->hasWeapon() || !units_dat::GroupFlags[selectedUnit->id].isBuilding)	//include armed building, exclude others buildings and include all other units
					DrawTargetedOrder(selectedUnit);

				DrawQueuedOrders(selectedUnit);

				if(
					selectedUnit->status & UnitStatus::GroundedBuilding ||
					selectedUnit->id == UnitId::ProtossCarrier ||
					selectedUnit->id == UnitId::Hero_Gantrithor ||
					selectedUnit->id == UnitId::ProtossReaver ||
					selectedUnit->id == UnitId::Hero_Warbringer ||
					selectedUnit->id == UnitId::TerranSiegeTankSiegeMode ||
					selectedUnit->id == UnitId::Hero_EdmundDukeSiegeMode ||
					(selectedUnit->id == UnitId::ZergLurker && selectedUnit->status & UnitStatus::Burrowed) ||
					selectedUnit->mainOrderId == OrderId::AtkMoveEP ||
					selectedUnit->mainOrderId == OrderId::MedicHeal1 ||
					selectedUnit->mainOrderId == OrderId::HealMove
				)
					TryDrawMinWeaponRange(selectedUnit);

				if (selectedUnit->status & UnitStatus::Cloaked) {
					coords.left = MIN(coords.left, MAX(selectedUnit->getLeft() - MAX_SIGHT_RANGE * 32, 0) );
					coords.top = MIN(coords.top, MAX(selectedUnit->getTop() - MAX_SIGHT_RANGE * 32, 0) );
					coords.right = MAX(coords.right, MIN(selectedUnit->getRight() + MAX_SIGHT_RANGE * 32,  mapTileSize->width * 32 - 32) );
					coords.bottom = MAX(coords.bottom, MIN(selectedUnit->getBottom() + MAX_SIGHT_RANGE * 32, mapTileSize->height * 32 - 32) );
				}

			}

		}

		selectedUnit = *activePortraitUnit;

		TryDrawFlyingBuildingPathToLandingPoint();

		if (coords.right != 0) {

			CUnit** detectors_search;

			detectors_search = getAllUnitsInBounds(&coords);

			while(*detectors_search != NULL) {

				if (
					(*detectors_search)->isTargetEnemy(selectedUnit) &&
					(*detectors_search)->canDetect() &&
					(*detectors_search)->isVisibleTo(*LOCAL_HUMAN_ID)
				)
					DrawDetectorRange(*detectors_search);

				detectors_search++;					//go on next unit of the list (or null)

			} //while(current_unit != NULL)

			//reload the previous temporary unit list from before the call to getAllUnitsInBounds
			*tempUnitsListCurrentArrayCount = tempUnitsListArraysCountsListLastIndex[*tempUnitsListArraysCountsListLastIndex];
			*tempUnitsListArraysCountsListLastIndex = *tempUnitsListArraysCountsListLastIndex - 1;

		}

	}

}